// Copyright 2011 Julian Phillips.  All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package py

// #include "utils.h"
// static inline int complexCheck(PyObject *o) { return PyComplex_Check(o); }
//import "C"

import (
	"fmt"
	"unsafe"
)

type Complex struct {
	AbstractObject
	NumberProtocol
	o _PyComplexObject
}

// ComplexType is the Type object that represents the Complex type.
var ComplexType = (*Type)(unsafe.Pointer(&*_PyComplex_Type))

func complexCheck(obj Object) bool {
	return _complexCheck(c(obj)) != 0
}

func newComplex(obj *_PyObject) *Complex {
	return (*Complex)(unsafe.Pointer(obj))
}

func NewComplex(v complex128) (*Complex, error) {
	ret := _PyComplex_FromDoubles(float64(real(v)), float64(imag(v)))
	if ret == nil {
		return nil, exception()
	}
	return newComplex(ret), nil
}

func (o *Complex) Complex128() complex128 {
	r := float64(_PyComplex_RealAsDouble(c(o)))
	i := float64(_PyComplex_ImagAsDouble(c(o)))
	return complex(r, i)
}

func (c *Complex) String() string {
	if c == nil {
		return "<nil>"
	}
	return fmt.Sprintf("%v", c.Complex128())
}
