// Copyright 2011 Julian Phillips.  All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package py

// #include "utils.h"
// static inline int floatCheck(PyObject *o) { return PyFloat_Check(o); }
//import "C"

import (
	"fmt"
	"unsafe"
)

type Float struct {
	AbstractObject
	NumberProtocol
	o _PyFloatObject
}

// FloatType is the Type object that represents the Float type.
var FloatType = (*Type)(unsafe.Pointer(&*_PyFloat_Type))

func floatCheck(obj Object) bool {
	return _floatCheck(c(obj)) != 0
}

func newFloat(obj *_PyObject) *Float {
	return (*Float)(unsafe.Pointer(obj))
}

func NewFloat(v float64) (*Float, error) {
	ret := _PyFloat_FromDouble(float64(v))
	if ret == nil {
		return nil, exception()
	}
	return newFloat(ret), nil
}

func NewFloatString(v string) (*Float, error) {
	s, err := NewString(v)
	if err != nil {
		return nil, err
	}
	defer s.Decref()
	ret := _PyFloat_FromString(c(s), nil)
	if ret == nil {
		return nil, exception()
	}
	return newFloat(ret), nil
}

func (f *Float) Float64() float64 {
	return float64(_PyFloat_AsDouble(c(f)))
}

func (f *Float) String() string {
	if f == nil {
		return "<nil>"
	}
	return fmt.Sprintf("%v", f.Float64())
}
