// Copyright 2011 Julian Phillips.  All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package py

// #include "utils.h"
// static inline int typeCheck(PyObject *o) { return PyType_Check(o); }
// static inline int typeCheckE(PyObject *o) { return PyType_CheckExact(o); }
// static inline PyObject *typeAlloc(PyObject *t, Py_ssize_t n) {
//    return ((PyTypeObject *)t)->tp_alloc((PyTypeObject *)t, n);
// }
// static inline int typeInit(PyObject *t, PyObject *o, PyObject *a, PyObject *k) {
//    return ((PyTypeObject *)t)->tp_init(o, a, k);
// }
//import "C"

import "unsafe"

type Type struct {
	AbstractObject
	o _PyTypeObject
}

// TypeType is the Type object that represents the Type type.
var TypeType = (*Type)(unsafe.Pointer(&*_PyType_Type))

func typeCheck(obj Object) bool {
	if obj == nil {
		return false
	}
	return _typeCheck(c(obj)) != 0
}

func newType(obj *_PyObject) *Type {
	return (*Type)(unsafe.Pointer(obj))
}

func (t *Type) Alloc(n int64) (Object, error) {
	ret := _typeAlloc(c(t), int64(n))
	return obj2ObjErr(ret)
}

func (t *Type) Init(obj Object, args *Tuple, kw *Dict) error {
	ret := _typeInit(c(t), c(obj), c(args), c(kw))
	if ret < 0 {
		return exception()
	}
	return nil
}

func (t *Type) String() string {
	pyS := _PyObject_Str(c(t))
	if pyS == nil {
		return "<unknown type>"
	}
	return _GoString(_PyString_AsString(pyS))
}

// CheckExact returns true when "t" is an actual Type object, and not some form
// of subclass.
func (t *Type) CheckExact() bool {
	return _typeCheckE(c(t)) == 1
}

// PyType_ClearCache : TODO - ???

// Modified should be called after the attributes or base class of a Type have
// been changed.
func (t *Type) Modified() {
	_PyType_Modified(&t.o)
}

// HasFeature returns true when "t" has the feature in question.
func (t *Type) HasFeature(feature uint32) bool {
	return (t.o.tp_flags & int64(feature)) != 0
}

// IsGc returns true if the type "t" supports Cyclic Garbage Collection.
func (t *Type) IsGc() bool {
	return t.HasFeature(TPFLAGS_HAVE_GC)
}

// IsSubtype returns true if "t" is a subclass of "t2".
func (t *Type) IsSubtype(t2 *Type) bool {
	return _PyType_IsSubtype(&t.o, &t2.o) == 1
}

// PyType_GenericAlloc : This is an internal function, which we should not need
// to expose.

// PyType_GenericNew : Another internal function we don't need to expose.

// PyType_Ready : This function is wrapped (along with a lot of other
// functionality) in the Create method of the Class stuct.
